searchNodes=[{"ref":"Commandex.html","title":"Commandex","type":"module","doc":"Defines a command struct. Commandex structs are a loose implementation of the command pattern, making it easy to wrap parameters, data, and errors into a well-defined struct. Example A fully implemented command module might look like this: defmodule RegisterUser do import Commandex command do param :email param :password data :password_hash data :user pipeline :hash_password pipeline :create_user pipeline :send_welcome_email end def hash_password(command, %{password: nil} = _params, _data) do command |&gt; put_error(:password, :not_given) |&gt; halt() end def hash_password(command, %{password: password} = _params, _data) do put_data(command, :password_hash, Base.encode64(password)) end def create_user(command, %{email: email} = _params, %{password_hash: phash} = _data) do %User{} |&gt; User.changeset(%{email: email, password_hash: phash}) |&gt; Repo.insert() |&gt; case do {:ok, user} -&gt; put_data(command, :user, user) {:error, changeset} -&gt; command |&gt; put_error(:repo, changeset) |&gt; halt() end end def send_welcome_email(command, _params, %{user: user}) do Mailer.send_welcome_email(user) command end end The command/1 macro will define a struct that looks like: %RegisterUser{ success: false, halted: false, errors: %{}, params: %{email: nil, password: nil}, data: %{password_hash: nil, user: nil}, pipelines: [:hash_password, :create_user, :send_welcome_email] } As well as two functions: &amp;RegisterUser.new/1 &amp;RegisterUser.run/1 &amp;new/1 parses parameters into a new struct. These can be either a keyword list or map with atom/string keys. &amp;run/1 takes a command struct and runs it through the pipeline functions defined in the command. Functions are executed in the order in which they are defined. If a command passes through all pipelines without calling halt/1, :success will be set to true. Otherwise, subsequent pipelines after the halt/1 will be ignored and :success will be set to false. Example %{email: &quot;example@example.com&quot;, password: &quot;asdf1234&quot;} |&gt; RegisterUser.new() |&gt; RegisterUser.run() |&gt; case do %{success: true, data: %{user: user}} -&gt; # Success! We&#39;ve got a user now %{success: false, errors: %{password: :not_given}} -&gt; # Respond with a 400 or something %{success: false, errors: _error} -&gt; # I&#39;m a lazy programmer that writes catch-all error handling end"},{"ref":"Commandex.html#command/1","title":"Commandex.command/1","type":"macro","doc":"Defines a command struct with params, data, and pipelines."},{"ref":"Commandex.html#data/1","title":"Commandex.data/1","type":"macro","doc":"Defines a command data field. Data field values are created and set as pipelines are run. Set one with put_data/3. command do # ...params data :password_hash data :user # ...pipelines end"},{"ref":"Commandex.html#halt/1","title":"Commandex.halt/1","type":"function","doc":"Halts a command pipeline. Any pipelines defined after the halt will be ignored. If a command finishes running through all pipelines, :success will be set to true. def hash_password(command, %{password: nil} = _params, _data) do command |&gt; put_error(:password, :not_supplied) |&gt; halt() end"},{"ref":"Commandex.html#param/1","title":"Commandex.param/1","type":"macro","doc":"Defines a command parameter field. Parameters are supplied at struct creation, before any pipelines are run. command do param :email param :password # ...data # ...pipelines end"},{"ref":"Commandex.html#pipeline/1","title":"Commandex.pipeline/1","type":"macro","doc":"Defines a command pipeline. Pipelines are functions executed against the command, in the order in which they are defined. For example, two pipelines could be defined: pipeline :check_valid_email pipeline :create_user Which could be mentally interpreted as: command |&gt; check_valid_email() |&gt; create_user() A pipeline function must be of arity three (command, params, data), but can be defined multiple ways: pipline :create_user # The name of a function inside the command&#39;s module pipeline &amp;YourModule.create_user/3 pipeline {YourModule, :create_user}"},{"ref":"Commandex.html#put_data/3","title":"Commandex.put_data/3","type":"function","doc":"Sets a data field with given value. Define a data field first: data :password_hash Set the password pash in one of your pipeline functions: def hash_password(command, %{password: password} = _params, _data) do # Better than plaintext, I guess put_data(command, :password_hash, Base.encode64(password)) end"},{"ref":"Commandex.html#put_error/3","title":"Commandex.put_error/3","type":"function","doc":"Sets error for given key and value. :errors is a map. Putting an error on the same key will overwrite the previous value. def hash_password(command, %{password: nil} = _params, _data) do command |&gt; put_error(:password, :not_supplied) |&gt; halt() end"},{"ref":"Commandex.html#t:command/0","title":"Commandex.command/0","type":"type","doc":""}]